@@ -1,8 +1,7 @@
 use crate::{
     event::{Event, EventTrigger},
-    ShutdownToken,
+    BoxedError, ShutdownToken,
 };
-use anyhow::Result;
 use std::future::Future;
 use tokio::task::{JoinError, JoinHandle};
 
@@ -22,7 +21,7 @@ impl Drop for SubsystemRunner {
 
 impl SubsystemRunner {
     async fn handle_subsystem(
-        mut inner_joinhandle: JoinHandle<Result<()>>,
+        mut inner_joinhandle: JoinHandle<Result<(), BoxedError>>,
         shutdown_token: ShutdownToken,
         local_shutdown_token: ShutdownToken,
         name: String,
@@ -63,7 +62,7 @@ impl SubsystemRunner {
         }
     }
 
-    pub fn new<Fut: 'static + Future<Output = Result<()>> + Send>(
+    pub fn new<Fut: 'static + Future<Output = Result<(), BoxedError>> + Send>(
         name: String,
         shutdown_token: ShutdownToken,
         local_shutdown_token: ShutdownToken,
