@@ -1,12 +1,11 @@
 use std::future::Future;
 use std::sync::Arc;
 
-use anyhow::Result;
-
 use super::NestedSubsystem;
 use super::SubsystemData;
 use super::SubsystemHandle;
 use crate::runner::SubsystemRunner;
+use crate::BoxedError;
 use crate::PartialShutdownError;
 use crate::ShutdownToken;
 
@@ -54,8 +53,8 @@ impl SubsystemHandle {
     /// ```
     ///
     pub fn start<
-        Err: Into<anyhow::Error>,
-        Fut: 'static + Future<Output = core::result::Result<(), Err>> + Send,
+        Err: Into<BoxedError>,
+        Fut: 'static + Future<Output = Result<(), Err>> + Send,
         S: 'static + FnOnce(SubsystemHandle) -> Fut + Send,
     >(
         &self,
