@@ -1,11 +1,14 @@
 use anyhow::anyhow;
 use env_logger;
 use tokio::time::{sleep, timeout, Duration};
-use tokio_graceful_shutdown::{PartialShutdownError, SubsystemHandle, Toplevel};
+use tokio_graceful_shutdown::{
+    GracefulShutdownError, PartialShutdownError, SubsystemHandle, Toplevel,
+};
 
 mod common;
 use common::event::Event;
 
+use std::error::Error;
 use std::sync::Once;
 
 static INIT: Once = Once::new();
@@ -19,7 +22,8 @@ fn setup() {
 }
 
 /// Wrapper function to simplify lambdas
-type AnyhowResult = Result<(), anyhow::Error>;
+type BoxedError = Box<dyn Error + Sync + Send>;
+type BoxedResult = Result<(), BoxedError>;
 
 #[tokio::test]
 async fn normal_shutdown() {
@@ -28,7 +32,7 @@ async fn normal_shutdown() {
     let subsystem = |subsys: SubsystemHandle| async move {
         subsys.on_shutdown_requested().await;
         sleep(Duration::from_millis(200)).await;
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let toplevel = Toplevel::new().start("subsys", subsystem);
@@ -40,7 +44,7 @@ async fn normal_shutdown() {
             shutdown_token.shutdown();
         },
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .handle_shutdown_requests(Duration::from_millis(400))
                 .await;
             assert!(result.is_ok());
@@ -55,7 +59,7 @@ async fn shutdown_timeout_causes_error() {
     let subsystem = |subsys: SubsystemHandle| async move {
         subsys.on_shutdown_requested().await;
         sleep(Duration::from_millis(400)).await;
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let toplevel = Toplevel::new().start("subsys", subsystem);
@@ -71,6 +75,7 @@ async fn shutdown_timeout_causes_error() {
                 .handle_shutdown_requests(Duration::from_millis(200))
                 .await;
             assert!(result.is_err());
+            assert_eq!(result, Err(GracefulShutdownError::ShutdownTimeout))
         },
     );
 }
@@ -79,7 +84,7 @@ async fn shutdown_timeout_causes_error() {
 async fn subsystem_finishes_with_success() {
     setup();
 
-    let subsystem = |_| async { AnyhowResult::Ok(()) };
+    let subsystem = |_| async { BoxedResult::Ok(()) };
 
     let (toplevel_finished, set_toplevel_finished) = Event::create();
 
@@ -88,7 +93,7 @@ async fn subsystem_finishes_with_success() {
 
     tokio::join!(
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .handle_shutdown_requests(Duration::from_millis(100))
                 .await;
             set_toplevel_finished();
@@ -120,7 +125,7 @@ async fn subsystem_finishes_with_error() {
 
     tokio::join!(
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .handle_shutdown_requests(Duration::from_millis(100))
                 .await;
             set_toplevel_finished();
@@ -145,12 +150,13 @@ async fn subsystem_receives_shutdown() {
     let subsys = |subsys: SubsystemHandle| async move {
         subsys.on_shutdown_requested().await;
         set_subsys_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let toplevel = Toplevel::new().start("subsys", subsys);
     let shutdown_token = toplevel.get_shutdown_token().clone();
-    let result = tokio::spawn(toplevel.handle_shutdown_requests(Duration::from_millis(100)));
+    let result =
+        tokio::spawn(toplevel.handle_shutdown_requests::<BoxedError>(Duration::from_millis(100)));
 
     sleep(Duration::from_millis(100)).await;
     assert!(!subsys_finished.get());
@@ -177,18 +183,19 @@ async fn nested_subsystem_receives_shutdown() {
     let nested_subsystem = |subsys: SubsystemHandle| async move {
         subsys.on_shutdown_requested().await;
         set_subsys_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let subsystem = |subsys: SubsystemHandle| async move {
         subsys.start("nested", nested_subsystem);
         subsys.on_shutdown_requested().await;
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let toplevel = Toplevel::new().start("subsys", subsystem);
     let shutdown_token = toplevel.get_shutdown_token().clone();
-    let result = tokio::spawn(toplevel.handle_shutdown_requests(Duration::from_millis(100)));
+    let result =
+        tokio::spawn(toplevel.handle_shutdown_requests::<BoxedError>(Duration::from_millis(100)));
 
     sleep(Duration::from_millis(100)).await;
     assert!(!subsys_finished.get());
@@ -215,7 +222,7 @@ async fn nested_subsystem_error_propagates() {
     let subsystem = move |subsys: SubsystemHandle| async move {
         subsys.start("nested", nested_subsystem);
         subsys.on_shutdown_requested().await;
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let (toplevel_finished, set_toplevel_finished) = Event::create();
@@ -225,7 +232,7 @@ async fn nested_subsystem_error_propagates() {
 
     tokio::join!(
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .handle_shutdown_requests(Duration::from_millis(100))
                 .await;
             set_toplevel_finished();
@@ -252,7 +259,7 @@ async fn panic_gets_handled_correctly() {
     let subsystem = move |subsys: SubsystemHandle| async move {
         subsys.start::<anyhow::Error, _, _>("nested", nested_subsystem);
         subsys.on_shutdown_requested().await;
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let (toplevel_finished, set_toplevel_finished) = Event::create();
@@ -262,7 +269,7 @@ async fn panic_gets_handled_correctly() {
 
     tokio::join!(
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .handle_shutdown_requests(Duration::from_millis(100))
                 .await;
             set_toplevel_finished();
@@ -291,7 +298,7 @@ async fn subsystem_can_request_shutdown() {
         subsys.request_shutdown();
         subsys.on_shutdown_requested().await;
         set_subsys_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let (toplevel_finished, set_toplevel_finished) = Event::create();
@@ -301,7 +308,7 @@ async fn subsystem_can_request_shutdown() {
 
     tokio::join!(
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .handle_shutdown_requests(Duration::from_millis(100))
                 .await;
             set_toplevel_finished();
@@ -336,7 +343,7 @@ async fn shutdown_timeout_causes_cancellation() {
         subsys.on_shutdown_requested().await;
         sleep(Duration::from_millis(300)).await;
         set_subsys_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let (toplevel_finished, set_toplevel_finished) = Event::create();
@@ -346,7 +353,7 @@ async fn shutdown_timeout_causes_cancellation() {
 
     tokio::join!(
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .handle_shutdown_requests(Duration::from_millis(200))
                 .await;
             set_toplevel_finished();
@@ -386,7 +393,7 @@ async fn spawning_task_during_shutdown_causes_task_to_be_cancelled() {
     let nested = |_: SubsystemHandle| async move {
         sleep(Duration::from_millis(100)).await;
         set_nested_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let subsystem = move |subsys: SubsystemHandle| async move {
@@ -394,7 +401,7 @@ async fn spawning_task_during_shutdown_causes_task_to_be_cancelled() {
         sleep(Duration::from_millis(100)).await;
         subsys.start("Nested", nested);
         set_subsys_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let (toplevel_finished, set_toplevel_finished) = Event::create();
@@ -404,7 +411,7 @@ async fn spawning_task_during_shutdown_causes_task_to_be_cancelled() {
 
     tokio::join!(
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .handle_shutdown_requests(Duration::from_millis(500))
                 .await;
             set_toplevel_finished();
@@ -446,7 +453,7 @@ async fn double_panic_does_not_stop_graceful_shutdown() {
         subsys.on_shutdown_requested().await;
         sleep(Duration::from_millis(400)).await;
         set_subsys_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let subsys2 = |_subsys: SubsystemHandle| async move {
@@ -455,15 +462,15 @@ async fn double_panic_does_not_stop_graceful_shutdown() {
     };
 
     let subsys1 = move |subsys: SubsystemHandle| async move {
-        subsys.start::<anyhow::Error, _, _>("Subsys2", subsys2);
-        subsys.start::<anyhow::Error, _, _>("Subsys3", subsys3);
+        subsys.start::<BoxedError, _, _>("Subsys2", subsys2);
+        subsys.start::<BoxedError, _, _>("Subsys3", subsys3);
         subsys.on_shutdown_requested().await;
         sleep(Duration::from_millis(100)).await;
         panic!("Subsystem1 panicked!")
     };
 
-    let result = Toplevel::new()
-        .start::<anyhow::Error, _, _>("subsys", subsys1)
+    let result: BoxedResult = Toplevel::new()
+        .start::<BoxedError, _, _>("subsys", subsys1)
         .handle_shutdown_requests(Duration::from_millis(500))
         .await;
     assert!(result.is_err());
@@ -482,7 +489,7 @@ async fn destroying_toplevel_cancels_subsystems() {
         set_subsys_started();
         sleep(Duration::from_millis(100)).await;
         set_subsys_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     {
@@ -505,13 +512,13 @@ async fn destroying_toplevel_cancels_nested_toplevel_subsystems() {
         set_subsys_started();
         sleep(Duration::from_millis(100)).await;
         set_subsys_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let subsys1 = move |_subsys: SubsystemHandle| async move {
         Toplevel::new()
             .start("subsys2", subsys2)
-            .handle_shutdown_requests(Duration::from_millis(100))
+            .handle_shutdown_requests::<BoxedError>(Duration::from_millis(100))
             .await
     };
 
@@ -540,14 +547,14 @@ async fn partial_shutdown_request_stops_nested_subsystems() {
         set_subsys3_started();
         subsys.on_shutdown_requested().await;
         set_subsys3_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
     let subsys2 = move |subsys: SubsystemHandle| async move {
         set_subsys2_started();
         subsys.start("subsys3", subsys3);
         subsys.on_shutdown_requested().await;
         set_subsys2_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let subsys1 = move |subsys: SubsystemHandle| async move {
@@ -561,7 +568,7 @@ async fn partial_shutdown_request_stops_nested_subsystems() {
         set_subsys1_shutdown_performed();
         subsys.on_shutdown_requested().await;
         set_subsys1_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let toplevel = Toplevel::new();
@@ -569,7 +576,7 @@ async fn partial_shutdown_request_stops_nested_subsystems() {
 
     tokio::join!(
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .start("subsys", subsys1)
                 .handle_shutdown_requests(Duration::from_millis(500))
                 .await;
@@ -614,10 +621,10 @@ async fn partial_shutdown_panic_gets_propagated_correctly() {
         assert!(!subsys.local_shutdown_token().is_shutting_down());
 
         subsys.request_shutdown();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
-    let result = Toplevel::new()
+    let result: BoxedResult = Toplevel::new()
         .start("subsys", subsys1)
         .handle_shutdown_requests(Duration::from_millis(500))
         .await;
@@ -650,10 +657,10 @@ async fn partial_shutdown_error_gets_propagated_correctly() {
         assert!(!subsys.local_shutdown_token().is_shutting_down());
 
         subsys.request_shutdown();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
-    let result = Toplevel::new()
+    let result: BoxedResult = Toplevel::new()
         .start("subsys", subsys1)
         .handle_shutdown_requests(Duration::from_millis(500))
         .await;
@@ -672,7 +679,7 @@ async fn partial_shutdown_during_program_shutdown_causes_error() {
         set_nested_started();
         subsys.on_shutdown_requested().await;
         set_nested_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let subsys1 = move |subsys: SubsystemHandle| async move {
@@ -693,10 +700,10 @@ async fn partial_shutdown_during_program_shutdown_causes_error() {
         assert!(nested_started.get());
         assert!(nested_finished.get());
 
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
-    let result = Toplevel::new()
+    let result: BoxedResult = Toplevel::new()
         .start("subsys", subsys1)
         .handle_shutdown_requests(Duration::from_millis(500))
         .await;
@@ -715,7 +722,7 @@ async fn partial_shutdown_on_wrong_parent_causes_error() {
         set_nested_started();
         subsys.on_shutdown_requested().await;
         set_nested_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let subsys1 = move |subsys: SubsystemHandle| async move {
@@ -734,16 +741,16 @@ async fn partial_shutdown_on_wrong_parent_causes_error() {
             assert!(nested_started.get());
             assert!(nested_finished.get());
 
-            AnyhowResult::Ok(())
+            BoxedResult::Ok(())
         };
 
         subsys.start("wrong_parent", wrong_parent);
         subsys.on_shutdown_requested().await;
 
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
-    let result = Toplevel::new()
+    let result: BoxedResult = Toplevel::new()
         .start("subsys", subsys1)
         .handle_shutdown_requests(Duration::from_millis(500))
         .await;
@@ -763,13 +770,13 @@ async fn cloned_handles_can_spawn_nested_subsystems() {
     let nested_subsystem1 = |subsys: SubsystemHandle| async move {
         subsys.on_shutdown_requested().await;
         set_nested1_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let nested_subsystem2 = |subsys: SubsystemHandle| async move {
         subsys.on_shutdown_requested().await;
         set_nested2_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let subsystem = move |subsys: SubsystemHandle| async move {
@@ -778,7 +785,7 @@ async fn cloned_handles_can_spawn_nested_subsystems() {
         subsys_clone.start("nested2", nested_subsystem2);
         subsys_clone.on_shutdown_requested().await;
         set_subsys_finished();
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let toplevel = Toplevel::new().start("subsys", subsystem);
@@ -786,7 +793,7 @@ async fn cloned_handles_can_spawn_nested_subsystems() {
 
     tokio::join!(
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .handle_shutdown_requests(Duration::from_millis(200))
                 .await;
             set_toplevel_finished();
@@ -824,7 +831,7 @@ async fn shutdown_through_signal() {
     let subsystem = |subsys: SubsystemHandle| async move {
         subsys.on_shutdown_requested().await;
         sleep(Duration::from_millis(200)).await;
-        AnyhowResult::Ok(())
+        BoxedResult::Ok(())
     };
 
     let toplevel = Toplevel::new().catch_signals();
@@ -836,7 +843,7 @@ async fn shutdown_through_signal() {
             signal::kill(Pid::this(), Signal::SIGINT).unwrap();
         },
         async {
-            let result = toplevel
+            let result: BoxedResult = toplevel
                 .start("subsys", subsystem)
                 .handle_shutdown_requests(Duration::from_millis(400))
                 .await;
